#pragma rtGlobals=1		// Use modern global access method.
#pragma version = 1.0
#pragma IgorVersion=5.0

Menu "SimPost", dynamic
	
	"Open Project", OpenProject()
	help = {"Open new project file"}	

	Submenu "Current Project"
		help = {"Macros for selected project (see red arrow at the \"Data Browser\")"}	

		GetCurrentProjectMenuString()
	End
	
	Submenu "Browser Selection"
		help = {"\"Data Browser\" selection sub menu"}
	End
		
	Submenu "Export Graphics"
		help = {"Save graph windows as emf-file"}

			GraphsMenuItem("All Graphs"), ExportGraphs(WinList("*", ";", "WIN:1" ))
			help = {"Save all open graph as emf-file", "Not available because there are no graphs."}

			GraphsMenuItem("Top Graph"), ExportGraphs(WinName(0,1))
			help = {"Save top graph windows as emf-file", "Not available because there are no graphs."}
	
			GraphsMenuItem("Top Batch"), ExportGraphs(GetTopGraphBatchList())
			help = {"Save graph windows in top batch as emf-file", "Not available because there are no graphs."}	
	End

	Submenu "Browser Selection"
		BrowserSelectionMenuItem("Display/1"), DisplayBrowserSelection()
		help = {"Display project data", "Not available because there are no project waves selected."}	
		BrowserSelectionMenuItem("Display In One Graph/2"), DisplayInOneGraph()
		help = {"Display project data", "Not available because there are no project waves selected."}
		
		Submenu "In All Graphs"		
				GraphsMenuItem("Hide in all Graphs"), ModityTacesInGraphs("hideTrace", "1", WinList("*", ";","WIN:1"))
				help = {"", "Not available because there are no graphs."}
				GraphsMenuItem("Show in all Graphs"), ModityTacesInGraphs("hideTrace", "0", WinList("*", ";","WIN:1"))
				help = {"", "Not available because there are no graphs."}
		End
		
		Submenu "In Top Graph"
				GraphsMenuItem("Hide in Top Graph"), ModityTacesInGraphs("hideTrace", "1", WinName(0,1))
				help = {"", "Not available because there are no graphs."}
				GraphsMenuItem("Show in Top Graph"), ModityTacesInGraphs("hideTrace", "0", WinName(0,1))
				help = {"", "Not available because there are no graphs."}
		End

		Submenu "In Top Batch"
				GraphsMenuItem("Hide in Top Batch"), ModityTacesInGraphs("hideTrace", "1", GetTopGraphBatchList())
				help = {"", "Not available because there are no graphs."}
				GraphsMenuItem("Show in Top Batch"), ModityTacesInGraphs("hideTrace", "0", GetTopGraphBatchList())
				help = {"", "Not available because there are no graphs."}
		End
	End
				
	"-"
	
	"About SimPost..."
	help = {"About SimPost version 1.0"}
End

Function/S BrowserSelectionMenuItem(inItemString)

	String inItemString
	
	String theSelection = GetBrowserSelection(-1)
	
	// check for open data browser
	if (strlen(theSelection) > 0)

		return inItemString
	endif
	
	// no browser, no selection
	return "(" + inItemString
End

Function/S GraphsMenuItem(inItemString)

	String inItemString
		
	// check for one graph
	if (OneGraphExists() == 0)
		return "(" + inItemString
	endif
		
	return inItemString
End

//  proto type functions
Function/S ProjectMenu_protofunc()
	return ""
End

Function/S GetCurrentProjectMenuString()

	String theCurrentProject = GetCurrentProjectName()
	
	// project selected
	if (strlen(theCurrentProject) > 0)

		// get application	
		String theApplication = GetStandardProjectData(theCurrentProject, "Application")
	
		// get base application ending
		String theEnding = GetEnding(theApplication)
	
		// get function list
		String theFunctionList = FunctionList("ProjectMenu_" + theEnding,";","VALTYPE:4,NPARAMS:0")
		
		if (ItemsInList(theFunctionList) == 1)
		
			// function name
			String theFunctionName = StringFromList(0,theFunctionList)
		
			// get menu function ref
			FUNCREF ProjectMenu_protofunc GetProjectMenuString = $(theFunctionName)

			return GetProjectMenuString()
		else
			// no project menu defined
			return ""
		endif
	else
		return "(no project selected"
	endif
End

Function OpenProject()

	String theFunctionList = FunctionList("GetApplicationData_*", ";", "KIND:2,NPARAMS:0")
	
	Variable theNumberOfFunctions = ItemsInList(theFunctionList,";")

	String theProjectEndings = ""
	
	Variable i
	
	for (i= 0; i < theNumberOfFunctions; i+= 1)
	
		String theFunction = StringFromList(i, theFunctionList,";")
	
		String theEnding = StringFromList(1,theFunction,"_")
	
		theProjectEndings += theEnding + ";"
	endfor

	// TODO: set filter string

	Variable Refnum
	Open /R /D  /T= "????" /M= "select a project file" Refnum

	Variable theNumberOfItems = ItemsInList(S_fileName,".")

	String theSelEnding = StringFromList(theNumberOfItems-1,S_fileName,".")
	
	// check ending
	if (WhichListItem(theSelEnding,theProjectEndings) > -1)
		DoOpenProject(S_fileName)
	endif
End

Function/S DoOpenProject(inFileName)

	String inFileName
	
	if (strlen(inFileName) > 0)
		
		// get file name
		String theFileName = StringFromList(ItemsInList(inFileName,":")-1,inFileName,":")
	
		// get ending
		String theEnding = StringFromList(1,theFileName,".")
	
		// get function list
		String theFunctionList = FunctionList("*_" + theEnding,";",",")
	
		// get index
		Variable theFunctionIndex = WhichListItem("GetApplicationData_" + theEnding,theFunctionList)

		// known ending?
		if (theFunctionIndex > -1)

			// get function name
			String theFunctionName =  StringFromList(theFunctionIndex, theFunctionList)
		
			// get function path
			String theFunctionPath = FunctionPath(theFunctionName)
			
			// get marcro file
			String theMacroFile = ParseFilePath(0, theFunctionPath, ":", 1, 0)
		
			// remove ending
			theMacroFile = StringFromList(0, theMacroFile, ".")
		
			// default project data
			String theProjectKeyList = "Application=" + theMacroFile + ";LineWidth=1;ColorMap=Rainbow;BaseDataFolder=root:Projects:" + theMacroFile + ";isProjectFile=True;Type=General;GraphStyleFunction=ProjectGraphStyle;GraphExportFunction=ProjectGraphExport;"

			// build new path name
			String thePathName = UniqueName("DataPath", 12, 0)
		
			// remove file name
			String thePath = RemoveListItem(ItemsInList(inFileName,":")-1,inFileName,":") 
		
			// build new path
			NewPath /Q $thePathName , thePath
			
			// get function ref
			FUNCREF GetApplicationData_protofunc GetApplicationData= $(theFunctionName)
			
			// get application
			String theApplicationData = GetApplicationData()
			
			// build project name from filename
			String theProjectName = StringFromList(0,theFileName,".")
		
			if (strlen(theProjectName) > 28)
			
				// limit size
				theProjectName = theProjectName[0,27]
			endif
	
			// get unique win name
			String theWinName = UniqueName("Project", 10, 0)
	
			// get ID
			String theID = theWinName[7,strlen(theWinName)-1]

			// get open projects
			String theProjectList = GetProjectKeyList("isProjectFile=True","ProjectName")

			// check name
			if (WhichlistItem(theProjectName,theProjectList,";") > -1 )
			
				Variable theProjectNameID = 0
			
				do
					// inc counter
					theProjectNameID += 1
					
				while (WhichlistItem(theProjectName  + " " + num2str(theProjectNameID),theProjectList,";") > -1)
			
				// make it unique
				theProjectName += " " + num2str(theProjectNameID)
			endif
	
			// get line style from ID
			if (str2num(theID) < 17)
			
				String theLineStyle = theWinName[7,strlen(theWinName)-1]
			else
				theLineStyle = "0"
			endif
		
			// add key
			theProjectKeyList +="LineStyle=" + theLineStyle + ";"
		
			Variable theApplicationDataIndex
			
			// complete project key list
			for (theApplicationDataIndex = 0; theApplicationDataIndex < ItemsInList(theApplicationData); theApplicationDataIndex += 1)
			
				String theItem = StringFromList(theApplicationDataIndex, theApplicationData)
				String theKey = StringFromList(0, theItem,"=")	
				String theValue = StringFromList(1, theItem,"=")

				// replace in default data
				theProjectKeyList = ReplaceStringByKey(theKey,theProjectKeyList,theValue,"=",";")
			endfor
		
			// need base data folder to create project data folder
			String theBaseDataFolder = StringByKey("BaseDataFolder",theProjectKeyList,"=",";")
					
			// add keys
			theProjectKeyList += "WindowName=" + theWinName + ";PathName=" + thePathName + ";FileName=" + theFileName + ";DataFolder="+ theBaseDataFolder + ":"+ "'"+ theProjectName + "'" +";ProjectID=" + theID + ";ProjectName=" + theProjectName +";creationData=" + date() + ";creationTime=" + time() + ";" 
			
			// open notebook
			NewNotebook /N=$theWinName /K=3 /F=0 /V=0 as theProjectName
			
			// get application
			String theApplication = StringByKey("Application", theProjectKeyList, "=", ";")
			
			// ok?
			if (V_Flag == 0)
				
				// set project keys!
				setWindow $theWinName note = theProjectKeyList
				
				// check for additional opening procedure
				theFunctionIndex = WhichListItem("OpenProject_" + theEnding,theFunctionList,";")

				if (theFunctionIndex > -1)
				
					// get function ref
					FUNCREF OpenProject_protofunc OpenProject= $(StringFromList(theFunctionIndex,theFunctionList,";"))
			
					// do application dep. open procedure
					OpenProject(theProjectName)
				endif
			
				// set project file hook
				SetWindow $theWinName hook =  ProjectFileHook

				// get standard folder
				String theStandardFolder = GetStandardProjectData(theProjectName,"DataFolder")
	
				// set and create folder
				CreateProjectDF(theStandardFolder)
			endif	
		endif
		
		// make visible
		Notebook $theWinName , visible=1
		
		// bring project window to front
		DoWindow /F $theWinName
		
		SetCurrentProject(theProjectName)
		
		// return project name
		return theProjectName
	endif
	
	return ""
End

Function/S GetEnding(inApplication)
	
	String inApplication
	
	String theFunctionList = FunctionList("GetApplicationData_*",";","NPARAMS:0")

	String theEnding = ""

	Variable i
	
	// find ending of base application
	for (i =0; i < ItemsInList(theFunctionList) ; i+= 1)
			
		// get function name	
		String theFunctionName = StringFromList(i,theFunctionList)
				
		// get path		
		String theFunctionPath = FunctionPath(theFunctionName)		
				
		// get file name
		String theFileName = ParseFilePath(0, theFunctionPath, ":", 1, 0)	
		
		// remove ".ipf"
		String theApplication = StringFromlist(0, theFileName, ".")
		
		if (stringmatch(inApplication, theApplication))
		
			// get ending
			theEnding = StringFromlist(1 ,theFunctionName , "_")			
			break
		endif
	endfor

	// return string
	return  theEnding
End

Function ProjectFileHook(infoStr)

	String infoStr

	String event = StringByKey("EVENT", infoStr)
	
	String theWindow = StringByKey("WINDOW", infoStr)
		
	String theProjectName = GetProjectName(theWindow)
		
	String theApplication = GetStandardProjectData(theProjectName, "Application")	
		
	String theEnding = GetEnding(theApplication)	
		
	String theFunctionList = FunctionList("ProjectWinHook_*_" + theEnding,";","KIND:2,NPARAMS:1")	

	// build user hook function name
	String theHookFunctionName = "ProjectWinHook_" + event + "_" + theEnding

	// check for user hook
	if (WhichListItem(theHookFunctionName,theFunctionList) > -1)

		// call
		Execute /Q /Z  theHookFunctionName + "(\"" + infoStr + "\")" 
		
		// user hook
		return 1
	endif

	// no hook
	return 0
End

Function CreateProjectDF(inFolder)

	String inFolder
	
	Variable i
	
	// start
	String theFolder = StringFromList(0,inFolder,":")
	
	Variable theNumberOfFolders = ItemsInList(inFolder,":")
	
	for (i= 1; i < theNumberOfFolders ; i+= 1)
	
		// build folder name
		theFolder += ":" + StringFromList(i, inFolder, ":")
	
		// check if folder exists
		if (!DataFolderExists(theFolder))
			
			// create if not exists
			NewDataFolder $theFolder
		endif	
	endfor
End

Function/S GetStandardProjectData(inProjectName, inKey)

	// project name and key
	String inProjectName, inKey

	String theNotebooks = WinList("Project*",";","Win:16")

	Variable theNumberOfNotebooks = ItemsInList(theNotebooks)

	String theData = ""

	Variable i
	
	// notebook loop
	for (i = 0; i < theNumberOfNotebooks; i += 1)
	
		// get win name
		String theWinName = StringFromList(i,theNotebooks)
	
		GetWindow $theWinName , note

		String theProjectWindowFlag = StringBykey("isProjectFile",S_Value,"=",";")

		// project window?
		if (stringmatch(theProjectWindowFlag, "True"))
	
			String theProjectName = StringBykey("ProjectName",S_Value,"=",";")
	
			// project found
			if (stringmatch(theProjectName,inProjectName) == 1)
		
				theData = StringBykey(inKey,S_Value,"=",";") 
		
				if (strlen(theData) > 0)
					break
				endif
			endif
		endif	
	endfor

	// return data, no match returns empty string
	return theData
End

Function/S GetProjectName(inWinName)

	String inWinName
	
	return GetProjectKeyList("WindowName=" + inWinName, "ProjectName")
End

Function SendProjectCommand(inMatchkeyList, inCommandList)

	String inMatchkeyList, inCommandList
	
	// get windows
	String theWindowList = GetProjectKeyList(inMatchKeyList, "WindowName")
	
	Variable theNumberOfWindows = ItemsInList(theWindowList)
	
	Variable theWindowIndex
	
	// win loop
	for (theWindowIndex = 0; theWindowIndex < theNumberOfWindows; theWindowIndex += 1)
	
		// get window name
		String theWindowName = StringFromList(theWindowIndex,theWindowList)
		
		Variable theCommandIndex
		
		Variable theNumberOfCommands = ItemsInList(inCommandList)
		
		// command loop
		for (theCommandIndex = 0; theCommandIndex < theNumberOfCommands; theCommandIndex += 1)
		
			String theItem = StringFromList(theCommandIndex, inCommandList)
			
			if (ItemsInList(theItem,"#") == 2)
			
				// build command
				String theCommand = StringFromList(0,theItem,"#") + theWindowName + StringFromList(1,theItem,"#")
			else
				// no "#"
				theCommand = theItem
			endif
			
			// execute
			Execute /Q theCommand
		endfor
	endfor
End

Function/S GetProjectKeyList(inMatchKeyList, inOutKey)

	String inMatchKeyList, inOutKey
	
	// get all windows
	String theWinList = WinList("*",";","Win:127")

	Variable theNumberOfWindows = ItemsInList(theWinList)

	String theOutKeyList = ""

	// get number of match keys
	Variable theNumberOfMatchKeys = ItemsInList(inMatchKeyList)

	Variable theWindowIndex

	// window loop
	for (theWindowIndex =0; theWindowIndex < theNumberOfWindows ; theWindowIndex+= 1)
	
		// get window name
		String theWinName = StringFromList(theWindowIndex, theWinList)
	
		// get note
		GetWindow $theWinName , note	
		
		// store data
		String theNote = S_Value 
	
		Variable theKeyIndex
	
		// check key list
		for (theKeyIndex =0; theKeyIndex < theNumberOfMatchKeys ; theKeyIndex+= 1)
		
			// get full key (name=value)
			String theFullKey = StringFromList(theKeyIndex, inMatchKeyList)
		
			// get name
			String theKey = StringFromList(0, theFullKey,"=")
			
			// get value
			String theValue = StringFromList(1, theFullKey,"=")
			
			// get value of current window
			String theValueOfCurrentWindow = StringByKey(theKey, theNote,"=",";")
						
			// check match
			if (!stringmatch(theValueOfCurrentWindow,theValue))
				
				// set index
				theKeyindex = -1
				
				// no match, exit loop
				break
			endif
		endfor
		
		String theOutKey = StringByKey(inOutKey,theNote,"=")
		
		// check match
		if (theKeyIndex != -1 && strlen(theOutKey) > 0)
		
			// all keys matched, add win name to list
			theOutKeyList += theOutKey + ";" 
		endif
	endfor
	
	if (itemsInList(theOutKeyList) > 0)
		
		// remove last ";"
		theOutKeyList = theOutKeyList[0,strlen(theOutKeyList)-2] 
	endif
	
	// return window list
	return theOutKeyList
End

Function/S GetCurrentProjectName()

	String theProjects = GetProjectKeyList("isProjectFile=True", "ProjectName")
	
	String theDataFolder = GetDataFolder(-1)
	
	Variable theNumberOfProjects = ItemsInList(theProjects)
	
	if (theNumberOfProjects == 1)
	
		return StringFromList(0, theProjects)
	else
	
		Variable theProjectIndex
	
		for (theProjectIndex = 0; theProjectIndex < theNumberOfProjects; theProjectIndex += 1)

			String theProjectName = StringFromList(theProjectIndex, theProjects)
		
			String theProjectDF = GetStandardProjectData(theProjectName, "DataFolder")
	
			if (compareDataFolders(theProjectDF + ":", theDataFolder) == 1)
			
				// current project found!
				return theProjectName
			endif
		endfor	
	endif
	
	// no project found
	return ""
End

Function SetProjectKey(inMatchKeyList, inProperty)

	String inMatchKeyList, inProperty
	
	String theProjectWinList = GetProjectKeyList(inMatchKeyList,"WindowName")
	
	Variable theNumberOfProjectWindows = ItemsInList(theProjectWinList) 
	
	Variable i
	
	for (i = 0; i < theNumberOfProjectWindows; i += 1)
	
		String theProjectWindowName = StringFromList(i,theProjectWinList)
		
		// get note
		GetWindow $theProjectWindowName , note
	
		String theNewNote = ReplaceStringByKey(StringFromList(0,inProperty,"="),S_Value,StringFromList(1,inProperty,"="),"=")
	
		// update note
		SetWindow $theProjectWindowName , note = theNewNote
	endfor
End

Function AddProjectKey(inMatchKeyList, inKey)

	String inMatchKeyList, inKey
	
	String theProjectWinList = GetProjectKeyList(inMatchKeyList,"WindowName")
	
	Variable theNumberOfProjectWindows = ItemsInList(theProjectWinList)
	
	Variable i
	
	for (i = 0; i < theNumberOfProjectWindows; i += 1)
	
		String theProjectWindowName = StringFromList(i,theProjectWinList)
		
		// get note
		GetWindow $theProjectWindowName , note
	
		String theNewNote = S_Value + inKey
	
		// update note
		SetWindow $theProjectWindowName , note = theNewNote
	endfor
End

Function/S NewProjectWindow(inProjectName, inPropertyList, inWinType)

	String inProjectName, inPropertyList
	Variable inWinType
	
	String theApplication = GetStandardProjectData(inProjectName, "Application")
	
	String theExportFunction = GetStandardProjectData(inProjectName, "GraphExportFunction")
		
	String theWinName = UniqueName(theApplication + "_graph_", 6 , 0)
	
	switch (inWinType)
	
		case 1:
			// open new graph
			Display /K= 1 /N=$theWinName as ""
			SetWindow $theWinName , hook(modified) = ProjectGraphHook
			break

		case 7:
			// open new panel
			NewPanel /K= 1 /W=(636.75,215,866.25,530) /N=$theWinName as inProjectName
			break
		
		case 19:

			// open new graph
			Execute ("NewGizmo /N=" + theWinName  + " /T=\"" + inProjectName + "\"")
		
			break		
		endswitch
	
	// set note!
	SetWindow $theWinName , note = "ProjectName=" + inProjectName +";Application=" + theApplication + ";isProjectGraph=True;GraphExportFunction=" + theExportFunction + ";WindowName=" + theWinName + ";creationDate=" + date() +";creationTime=" + time() + ";" + inPropertyList + ";"
	
	// return window name
	return theWinName
End

Function BeforeFileOpenHook(refNum,fileName,pathName,type,creator,kind)

	Variable refNum,kind	
	String fileName,pathName,type,creator

	String theFunctionList = FunctionList("BeforeFileOpenHook_*",";","KIND:2,NPARAMS:6")	

	// build user hook function name
	String theHookFunctionName = "BeforeFileOpenHook_" + StringFromList(1,fileName,".")

	// cleanup
	KillStrings /Z S_waveNames

	// check for user hook
	if (WhichListItem(theHookFunctionName,theFunctionList) > -1)

		// call
		Execute /Q /Z  theHookFunctionName + "(" + num2str(refNum) + ",\"" + fileName + "\",\"" + pathName + "\",\"" + type + "\",\"" + creator + "\"," + num2str(kind) + ")" 
		
		// user hook
		return 1
	endif

	// no user hook
	return 0
End

//  proto type functions
Function/S GetApplicationData_protofunc()
	return "theApplication"
End

Function OpenProject_protofunc(inProjectName)
	String inProjectName
End

Function AboutSimPost()

	PauseUpdate; Silent 1		// building window...
	NewPanel /W=(441,100,810,381) /K=1 as "About SimPost ..."
	ModifyPanel cbRGB=(65534,65534,65534)
	SetDrawLayer UserBack
	DrawPICT 229,25,1,1,ProcGlobal#INA_Logo
	DrawText 50,43,"SimPost by Schaeffler KG"
	DrawText 50,66,"version: 1.0"
	DrawLine 21,126,348,126
	DrawText 50,174,"more information: www.ina.com"
	DrawText 50,155,"customer support: simpost@schaeffler.com"
	DrawLine 21,124,348,124
	DrawLine 21,190,348,190
	DrawLine 21,188,348,188
	DrawText 50,238,"91074 Herzogenaurach, Germany"
	DrawText 50,219," 2006 Schaeffler KG, Industriestrae 1-3"
	Button button0,pos={298,240},size={50,20},proc=closeAboutBox_proc,title="OK"
End

Picture INA_Logo
	ASCII85Begin
	M,6r;%14!\!!!!.8Ou6I!!!"b!!!!l#Qau+!$/uA6i[2e.s+3^F@1([@<?4%DBMnmD.N]ABco2c+AQ
	iu+>PW)2'=V43]&]T1,U1)0JP78UF?`9!!!!(F@f1q#Ls2'&dp;\\ATNj!!!=E86L!6!"/2q!"/2r6
	EE#2!!!!%B0f6X!!'9b$iFE:!!!Qc8OPjDGhVOi#uhU?(B%7`)f#PM/-:\1UBqjJ3&*nP3M8*YW"=o
	t?t+.cLbU8hZ7htb1G]n):?"%h/u")V&Mjnd),6(;&M]A9NDU<5DJUmt2nd`sn(4J,()e*YBNOlN4+
	3R_?WFkAhX14>Hh[Pl&d/76hrJU\].`)C))(KEqUu6[+"KJRV')5=YNT#T#=:r]X#B1)[k0Rap<`0=
	h\VeE9\S<NBt)#C&OIbm1t?1!^Jn,$/Q->*k/R5m07@iF"F2DQPUG9%\X2XYR]5$X(B<q_;Q?41E,6
	&EhkgYYPT2Uth_R^3qau?4o-b@UJ3%3=h#%tG/'<r?eZsg)e=$hLq(Efg\JTT3(HmG*G5e?,I/4]<U
	sPdC9]Oc5NUPF9n)#4.+7^O,H4GVD^ZdhO3Wf(2b'&j+3N\W+DoR[1I\W"&TIB\&$K1H+!;oLEZ04a
	SnW*+V\'*8?lY#Te<:nXZj=([@p+1m_Y;Om,g%NAS9;O?)Fm9s<$h2U,VZ1X6lkh!t0$9Q8,6)W\Ss
	f?>nV\bVM]]ZIVTr;9Q+-PX78M!BAB[BMnsZEpPB.M0XS;uKP[-JF@fsCH2Q+6P&iU<BR>Z*>&GT=D
	\,g`hB0Uj*(_n4p#XX'aViOVMHY53<NB]@&,MEo[GeSseB@j68A,D9OZ:.W>&<[F(1?amB*\Y)%m7m
	6qCX*n\I=:BYb"50U&__a6X/T;F74PL,<fRX[o%u-LH0TqM8=h6PD_)Uq1X&(B?^psl5r3808u#A'M
	eq2@d1L;+N'_1.ceg<a"g6<'J-:i+[o0E!R[[Rkq57L_CGDj;TK@/h42BN>jY]"SNjN,q^OEHUWesZ
	&?ZcBHD18l,%5h^CDDkAMi*59]76(NLW;g`PlQ=8R>]epT6n;)<7t^Pe<sXT/0qe^KQ(8G&/C2"XjR
	5b9^WWR'GR0i7VO\mKo+4b`IV@j<k`JD:&QKm'Pft4Ll>';+AJb;i"OTj)0MiH;LnuLMm(N2p*oTSk
	9tQ$*BO+QmN9sCVSj/*C'iF&n+VOJ6R<a^O?TNn*cLfFi`>D%'fe@3nEsH9Qh+/PQ8,_c^QhoC=i"'
	)cce>Wp7OUkL66u+4j7i-ObeMn?1]q!Qn^1gV6r`D;8G'7SqH=k'Z$pkSq0/$(L][=0/ERbF$R&Vh;
	]X?u3<-WSr,X^QU.PCCcDU^4itk8?-qY1?i*Va82eU_M\Gi88;S9:NA?:P&CGsr%N)h_i2WAndTaP-
	leSf8KpuCN_?M-so0'l+nF(q.%U#2dPGA"^'TZ2Ijb\/Eni7DqFWr20uB9eG8qo\MK+(!L:51q&ss,
	GKn:Q_XY-TQ<=5XI&/J*(!#F*07q7iRb6k<t&:OR:X"Q!eu1/A,B&:jj&J>hAfT/t2Enm!^ZRF4JpW
	,:'5D8-m(EronQ(EjJPBPTkgm)kF27YI?>K%4j>)R94N342cjg\;'Y2m#;PT0dU_-e52B++Q^T80:@
	+3*kZIA\UN\h"CA/!n*M4FhO0AcVFu&5=sf`MG("b2QiF4V/'%CLQ3&E?#jps(T`2k%K>[OA=qH;Ug
	lQT,@FSGD(Tm1<P_-"jNjmSCN'trldr_6L_]s0!+?H*+?nY/=cR;Vh@@SJA9kKuLN(@Wi='Pj:-"DY
	)<bt;16<jCpghQdU%n9R3Y^iU@o;9r4S%A!DN'YQ=NgXhEXJ?G[?LkPFY;K>_MMVptF(TmZ(dHQ0C>
	@OM*t>fQ.e!;gCSLaE"Ou--$CZO'n;2bK$uH5EESX@GN"q24=TdjLZ&h)N;Q6jF-uUhHJ(CLrK)mMi
	)ldX#YG#YK1)J"K_u4r8)uBbk0E'_$Ka"Ya&Fr\56N)L?SY`'"ZD4)VhL</4!lCJE&E_T.G"R1flHe
	7j.!Of\PPU9%_RSh/]'tHZ5(1K#Gl=PHqAi?QfdOq"!Q-X.!*q#NY%ZAbfSK1<fEA,b$%;3Q8.W)kF
	:Kt;W?XKn++4cj2o*oM<P->.$=(S@Mt^S?KEC>F5Y?E)WNVk^rrScVC.EbN#UiHDOk""=kg<*l_Zn!
	OhbN>4p$c^H(\5nE$o_QbMpk-9I4F.21VB?Z/l,T=@26:3D8_c:me.:VLX4NK4SA0L/t*_R872Q$U1
	d,AIZV%)XJBU<2"_68JAY!^S5*3O]8Era*AEH8K%3%,AZg(Nr1Z8U=PHe>2>>4QoeXZNcKK'%Fh[CG
	Q"*C\_.l*1YD,L`dT@bo>E)_V=7)n^U>=KQk37Q>RQ1#-`bWXRg9Ed76c1dFplVnO`l3o-<bVj#/!"
	U!3.$HX0\ik0[l0Hep^THB'tA>l-ueU*:t3g+kNmN/bPIVWZb$i.T"T@)]ob[djOT?f`/TK;#4Mh1M
	b`_*<`'3T!Bu85"cr!jO#i",iMup4";Eii5Zhm5Odrc91E8X0h5UG.2ZVcX<I&F[*Gg'Q2=F=i7A99
	llImY'k_S*cE'1YEq!rW?[?lcns';e.&=l-6p:j,N)/Y2,CJBe&gD?$N:u,SuJ.T`8H-Q3=/]ic@YF
	\m&]T[.B$QGHn4H4Nmb,0Yb<.o`,8L]p`@a=JMoK/7-Acs\^1B]D&P>^oh3;aKUJb!<[hjMP;&FM]*
	\-R,c/nrR90a4=sYNYr5CM1c0G7pNF[fqCsB91+VRt0M>LdC?)Q``te"pG=T94;=$jB]hbqRSFSB@&
	e?q!RNn-?9KIF7)Yg4`hUJ(iRqV;07H@r:4LhET1B'C:dMR&/^9Bdq9Nn"G[ks-Lrpu(42mMHKPEAm
	G(,;>XUi%608E]hjMP;V7f?l?CgWln7,3r6*p@6_/K\J(B<rc)N)&HM"q-K`4)4j3uf1MSj/B:[Pqu
	8.dXHRr*irg.+U.^?t-'QF34H0iUR*6^*(1%JYlKo^Rc-\0;`@M6#)[GL$+."f"JmGn8*KU\D=JTN!
	YBQrQ`?1=L)JZmJg38amY90F%'G78tIGH+$s=(\U<CX'4T"@MnO%Fo\I52DeOg84"e&#`7Qes6[s1D
	;HZotdFk;$8BHZU$-c-'?Zu,7d>G4IK-'M`q]]8##/frm;u*UB/p_cp[/N@RUp8M'E(OF.<Q3Qi#2Q
	qS1SGmAPZ*g=g*\@(&sA]if7.IXfG=61D:ni5)k].NVWuNF!ZO+Fe`9dlf!=t<]`QI[S_q(=X]Tho/
	;;">+1r*l^B7dJfAPQ$pXLSrM2r(7:(e4bO6I7an@+gTMmm&/XF>'E>MEenZp/"#HbECI7u+#8()%P
	u1M72Hi,V1riTWZGq9n\%JoQopjeq[(,N5$GAY1qd3K@/H%-SGt8`m&:^u!.\e\-<e$?EU@VR9?hc%
	iY-9"/u%.\YBqNh.Hl8.G!s(M>!Q@ab'b%NMutb^Kmj9o7(`faKe+<BL<4Yq(iQV`NnLkb+mQ=mH:q
	pI/t"BACKn_AM4j#ErN'>)Z\.p>@t-2NF`r/?.A?[b"*Gf`67:r9:^V+o=q:0Nr+(d)/APfb'MJl*?
	_0-PIQ<nP,PH0?:P:**FS\UR/^7:Unf0c)#d\3`^J,._0lZalG6nM<WEs@*Vr3U4U#?/)38?7)%L$=
	e=@ApA0t7S]p_`q#]hhrAMSrT#95G3<VkTa$V6J638!96Pg8I8h\\;mN+D4KtPsk.Q'O,e,&l=SqG7
	`ki#6d+*';haR'P5()3kY5'_9Dl#-E_n$Z.s9?(6dbEE]'h9"%9\rr-34S7T^<KaYR^4DFHbZZqJ3#
	Y?g5NMP66IZWFQ#!;2Ti?;S/BQqo#i-Ni#V&:]W=:0<:L!GeDq:]CiKYM;.i.X?,7XVaF8pV2Z=c,U
	\2`ZS.EH8hAk)pjKVk3e'70q!s%eI-(rG]1*M/8_&)!NLTqq0Fm]$ZL)Y"*;B2W!O<J0_OCM*h*Nir
	"@=#X/[2cM*BNe*l9HFVC*"h[eiR$=)`$<DViVUf5A1;>RD\G2/qca#Go`Ian=+-Hm;M.+.q\C+KMo
	%KTW;nKq9R_E$V/O5P+`()Sm^DtFrW4u)^q`p-JH2[DR/WA^6r%S/F`<fsEiJ/Z^+VL./+u,gpT(7Y
	^,;L*e'R6/YI=a`5MWEZR,L_*!L:U)L2kI-(8#"m271P1D%IS%6[LOIMNUIa3B`;&,T1KJV,W,jiXl
	@n[<,5BubT3;+2(nIdh0k$2qtD?bdO/Z<26b(`59+-X%R,XBp4s6k_t?F#UBu=D2QU<[)Tguo,RC+"
	0Gc+ORPd3u!iL7*L`A@Ys6)_-58)dXBrDn60Tk9`]K^8[,fEhHeJ5so_nb2Z#NflDYC>W^gg=#?[H:
	(K<8a!5c2&+.O]$m<4eV2GnbZVa4]JbF`\/4%:"o!Bn%DRce4OYc^`gkND\q_PObI9Ci-,'qA!8MQ*
	@m,NTi2P=FhpL4M!tiTO@TZY_lAu3=n0JkLV9tcp?ORsHDKk`>h/t=o,i&*8O5R^BbFu#0tWIWG1rE
	XQ1t.7*DUb,An*EVi[6/m"UUYO**D`JjZ/&r!45phn"9&u51DOD:[J7Hp<`Ug48N!LM[>E0Ndg-=Ur
	u)6%U=B-CIkJs3=S7[:jOF@$qS,XM+mC?ru-h*,o,>(4*0oL\^-bHVAmMZnN\7kmt_9I_o'0pcLVt*
	&fhOtGP2rci.n,$O`n_aeaO"oGli,p!lnFW@hT>aX`M,k#b=O+ZO["UWi+#*b0)1%\o3_["I6t'hgP
	OJ%6'-?6pD(I#NMG8n3bW:+Yh#WENV)$c9>h'0FQMqq:Bcs&<,`)54?q$T?.I*,k8.dd9=Q/e7H!R[
	ImM5#_FH/;"U$-at9FnXO%"W1geeb^`O69D(R8W*'p:^r3#=492\CSZX.WJs%0kPE:+TH"B)jgO07D
	sqF&/_bp@YpItFD2s*CfTh`9GqWG$2dJVn(br'I3\^N=9IK+Q_U72'IYhRbVBC_'#uhEAW;ok,Ff&3
	j``+-Ik%`8IqPMqO`:YmDlh_SpC0cb]DNq>;4HlIVR`H)*5k%4cD>Y@S/=V`V)t3Eh>i-=ct8Hs)8/
	Xh-TU/$9QP1^lJ_XURMmBTDHMcaG01(>c..9uF8>-\%/NI^W$ioj>=[O=puufiLs4[I5[`$61dlM[>
	=nH=X?pi.kjf6=Mi*.PVao;rI!hl=XZ+-h7Q@O9`"OQjX'-g%D]8_Q<F%KHKab((t\f^d7o8jWD&qF
	-Y8+TD],kmMRQ$f$$S6o&RMD\"(SWX+![*FL_+;YqjkU>agGPOI1G*goBF:B>_68\L9`BK%HX>qTpJ
	W6QX66@P%rBVb;*Sp'g.&PFqZ%I.]:^k(+)tc,?(&T?;X+2?EU9Ol-iCA-i3*lp"Cs,;l<`cX?eR!!
	#SZ:.26O@"J
	ASCII85End
End

Function closeAboutBox_proc(B_Struct)
	STRUCT WMButtonAction &B_Struct
	
	Variable theEvent =  B_Struct.eventCode
	
	if (theEvent == 1)
		String theWinName = B_Struct.win
	
		DoWindow /K $theWinName
	endif	
End

Function DisplayBrowserSelection()

	DoDisplayBrowserSelection("", "Unit;XDataType;AxisLabel;YAxisLabel;XAxisLabel;Type;NominalSpeed;SubTypes" )
End

Function DisplayInOneGraph()
	DoDisplayBrowserSelection("", "Unit;XDataType;AxisLabel;YAxisLabel;XAxisLabel;Type")
End

Function DoDisplayBrowserSelection(inAdditionalGraphKeyList, inMatchKeyList)

	String inAdditionalGraphKeyList, inMatchKeyList

	String theCurrentProjectName = GetCurrentProjectName()

	String theProjectList = GetProjectKeyList("isProjectFile=True", "ProjectName")
	
	Variable theNumberOfProjects = ItemsInList(theProjectList)
	
	String theSelectionList = GetSelectedWaves()
		 	 
	// filter project waves
	theSelectionList = ListWaveSearch(theSelectionList, "isProjectWave=True;")
		 	 		 	 
	Variable theNumberOfWaves = ItemsInList(theSelectionList)

	Variable theIndex

	// check for display functions and remove waves "time" and "engine speed"
	for (theIndex = ItemsInList(theSelectionList) - 1; theIndex >= 0; theIndex -= 1)
	
		// extract wave
		String theFullWaveName = StringFromList(theIndex, theSelectionList)
	
		// get wave note
		String theNote = Note($theFullWaveName)
	
		// get display function
		String theDisplayFunctionName = StringByKey("DisplayFunction", theNote, "=", ";") 
		
		// check Y-data key
		String theIsYDataKey = StringByKey("isYData", theNote, "=", ";")
		
		if (stringmatch(theIsYDataKey, "false") == 1)
		
			// remove item
			theSelectionList = RemoveFromList(theFullWaveName, theSelectionList)
		else
		
			if (strlen(theDisplayFunctionName) > 0)
		
				// get function ref
				FUNCREF DisplayProjectFFT_protofunc theDisplayFunction= $(theDisplayFunctionName)

				// call display function
				theDisplayFunction($theFullWaveName)

				// remove item
				theSelectionList = RemoveFromList(theFullWaveName, theSelectionList)
			endif	
		endif
	endfor
	
	// is there a selected wave
	if (itemsInList(theSelectionList) > 0)
	
		do
			//  get first item
			String theFirstWave = StringFromList(0,theSelectionList)
					
			// get match value list
			String theMatchValueList = GetWaveValueList(theFirstWave, inMatchKeyList)
	
			String theDisplayList = ""
	
			// get wave list
			for (theIndex = ItemsInList(theSelectionList)-1 ; theIndex > -1 ; theIndex -= 1)
			
				// get wave
				String theWaveName = StringFromList(theIndex, theSelectionList)
			
				String theValueList = GetWaveValueList(theWaveName, inMatchKeyList)
			
				// list ok
				if (stringmatch(theValueList, theMatchValueList))
			
					// add item to display list
					theDisplayList += theWaveName + ";"
				
					// remove wave from selection list	
					theSelectionList = RemoveFromList(theWaveName, theSelectionList)
				endif
			endfor

			// open new graph
			String theWinName = NewProjectWindow(theCurrentProjectName, "Type=BrrowserSelection;isPartOfSeries=False;" + inAdditionalGraphKeyList,1)

			// display waves
			DisplayProjectWaves(theWinName, theDisplayList)
			
			// check for graph style function
			String theGrapStyleFunctionName = GetStandardProjectData(theCurrentProjectName, "GraphStyleFunction")
			
			// execute function
			Execute /Q theGrapStyleFunctionName + "(\"" + theWinName + "\")"

		while(itemsInList(theSelectionList) > 0)
	endif	
End

Function Kill_If_Empty_and_cleanUp(inDataFolder)

	String inDataFolder
	
	Variable theLevels = ItemsInList(inDataFolder,":")

	Variable theFolderLevel
	
	for (theFolderLevel = theLevels-1; theFolderLevel > 0; theFolderLevel -= 1)
		
		// count objects
		Variable theTotalNumberOfObjects = 0
		Variable theTypeIndex
		
		// counter loop
		for (theTypeIndex = 1; theTypeIndex < 5; theTypeIndex += 1)
		
			// count objects
			Variable theNumberOfObjects = CountObjects(inDataFolder, theTypeIndex)
		
			// sum
			theTotalNumberOfObjects += theNumberOfObjects
		endfor
		
		// kill if empty
		if (theTotalNumberOfObjects == 0)
			
			// kill empty folder
			KillDataFolder $inDataFolder
			
			inDataFolder = RemoveListItem(theFolderLevel,inDataFolder,":")
		else
			break
		endif
	endfor
End

Function/S GetSelectedWaves()

	String theSelectionList = ""

	String theBrowserInfo = GetBrowserSelection(-1)

	if (strlen(theBrowserInfo) > 0)

		Variable theIndex = 0
		do
	
			// get line
			String theSelection = GetBrowserSelection (theIndex)

			// wave
			if (exists(theSelection) == 1)

				// add item
				theSelectionList += theSelection + ";"
			elseif(DataFolderExists(theSelection) && strlen(theSelection) > 0 && stringmatch("root", theSelection) == 0)
						
				String theNewWaves = DoRecursiveWaveSearch(theSelection + ":", "isProjectWave=True")
			
				Variable theNumberOfWaves = ItemsInList(theNewWaves)
			
				Variable i
				for (i=0; i < theNumberOfWaves; i+= 1)
			
					String theWave = StringFromList(i, theNewWaves)
				
					if (WhichListItem(theWave, theSelectionList) == -1)
						theSelectionList += theWave + ";"
					endif
				endfor
			endif
		
			// inc counter
			theIndex += 1
		
		while (strlen(theSelection) > 0)
	endif
	
	return theSelectionList
End

Function/S GetTopGraphBatchList()

	String theBatchGraphList = ""

	String theTopGraph = WinName(0, 1)

	// top graph exists?
	if (strlen(theTopGraph) > 0)

		GetWindow $theTopGraph, wsize

	 	Variable theTopLeft = V_left, theTopRight =V_right, theTopTop = V_top, theTopBottom = V_bottom

		String theGraphList = WinList("*", ";","WIN:1")

		Variable theNumberOfGraphWindows = ItemsInList(theGraphList)
	
		Variable theGraphIndex
	
		for (theGraphIndex = 0; theGraphIndex < theNumberOfGraphWindows; theGraphIndex += 1)

			String theGraphName = StringFromList(theGraphIndex, theGraphList)

			GetWindow $theGraphName, wsize
		
			if (isOnBatch(theTopLeft, V_left, theTopRight, V_right, theTopTop, V_top, theTopBottom, V_bottom) == 1)
		
				theBatchGraphList += theGraphName + ";"
			endif
		endfor
	
		return theBatchGraphList
	endif
	
	// no graphs, no batch
	return ""
End

Function isOnBatch(inBatchLeft, inLeft, inBatchRight, inRight, inBatchTop, inTop, inBatchBottom, inBottom)

	Variable inBatchLeft, inLeft, inBatchRight, inRight, inBatchTop, inTop, inBatchBottom, inBottom

	if (inBatchLeft >= inLeft && inBatchRight <= inRight && inBatchTop >= inTop && inBatchBottom <= inBottom)

		return 1
	else
		return 0
	endif
End

//  proto type functions
Function DisplayProjectFFT_protofunc(inWave)
	Wave inWave
End

Function/S GetWaveKey(inWaveName, inKey)

	String inWaveName, inKey

	// get note
	String theNote = note($inWaveName)

	// return key
	return StringByKey(inKey, theNote, "=", ";")
End

Function/S GetWaveValueList(inWaveName, inKeyList)

	String inWaveName, inKeyList

	// get note
	String theNote = note($inWaveName)

	String theValueList = ""
	
	Variable theNumberOfKeys = ItemsInList(inKeyList)
	
	Variable i
	
	for (i =0; i < theNumberOfKeys; i += 1)
	
		// get key
		String theKey = StringFromList(i, inKeyList)
	
		// store value
		theValueList += StringByKey(theKey, theNote, "=", ";") + ";"
	endfor
	
	// return values
	return theValueList
End

Function DisplayProjectWaves(inWinName, inFullWaveNames)

	String inWinName, inFullWaveNames
	
	String theLegendString = ""
	
	// wave loop
	Variable theWaveIndex
	
	Variable theNumberOfWaves = ItemsInList(inFullWaveNames)
	
	for (theWaveIndex = 0 ; theWaveIndex < theNumberOfWaves; theWaveIndex += 1)
	
		// get full wave name from list
		String theFullWaveName = StringFromList(theWaveIndex, inFullWaveNames, ";")
	
		String theDataFolder = ParseFilePath(1, theFullWaveName, ":", 1, 0)
	
		// wave name
		String theWaveName = ParseFilePath(0, theFullWaveName, ":", 1, 0)
	
		// get wave note
		String theNote = note($theFullWaveName)
	
		// get wave color
		String theColor = StringByKey("Color", theNote,"=",";")

		// get line width
		String theLineWidth = StringByKey("LineWidth", theNote,"=",";")
	
		// line style
		String theLineStyle = StringByKey("LineStyle", theNote,"=",";")
	
		// get x data
		String theXWaveName = StringByKey("XWaveName", theNote,"=",";")

		// get x data
		String theProjectName = StringByKey("ProjectName", theNote,"=",";")

		String theXWaves = GetProjectWaveList(theProjectName, "WaveName="+ theXWaveName + ";")

		String theXWave = StringFromList(0,theXWaves)

		// get standard marker
		String theMarker =  StringByKey("Marker", theNote,"=",";")

		// check x-wave
		if (strlen(theXWaveName) > 0)

			// append
			Execute /Q "AppendToGraph /W=" + inWinName + "/C=(" + theColor+ ") " + theFullWaveName + " vs " + theXWave
		else

			// append
			Execute /Q "AppendToGraph /W=" + inWinName + "/C=(" + theColor+ ") " + theFullWaveName
		endif
		// get trace names
		String theTraceNames = TraceNameList( inWinName, ";" , 1)

		// get last trace
		String theTraceName = StringFromList(ItemsInList(theTraceNames) -1, theTraceNames) 

		String theLegendText = StringByKey("Legend", theNote , "=" , ";")
		
		// set line style
		Execute "ModifyGraph /W=" + inWinName + " lstyle(" + theTraceName + ")=" + theLineStyle
		
		// set line width
		Execute "ModifyGraph /W=" + inWinName + " lsize(" + theTraceName + ")=" + theLineWidth
	
		// marker?
		if (strlen(theMarker) > 0) 

			Execute "ModifyGraph /W=" + inWinName + " mode=4,textMarker(" + theTraceName + ")={\"" + theMarker + "\",\"default\",0,0,5,0.00,0.00}"
		endif
	endfor	
End

Function /S DoRecursiveWaveSearch(inDataFolder, inMatchKeyList)

	String inDataFolder, inMatchKeyList

	Variable i

	// get waves in current folder
	Variable theNumberOfWaves = CountObjects(inDataFolder, 1)
	
	Variable theNumberofMatchKeys =  itemsInList(inMatchKeyList)
	
	// wave loop
	Variable theWaveindex
	
	String theWaveList = ""
	
	for (theWaveindex = 0; theWaveindex < theNumberOfWaves; theWaveindex += 1)

		// get wave name
		String theWaveName = GetIndexedObjName(inDataFolder,1,theWaveIndex)
		
		String theFullWaveName = inDataFolder + "'" + theWaveName + "'"
		
		// build attribute string
		String theWaveNote = "WaveName=" + theWaveName + ";"
		theWaveNote += "XUnit=" + WaveUnits($theFullWaveName, 0 ) + ";"  
		theWaveNote +=	"YUnit=" + WaveUnits($theFullWaveName, 1 ) + ";"	
		theWaveNote +=	"ZUnit=" + WaveUnits($theFullWaveName, 2 ) + ";"
		theWaveNote += "TUnit=" + WaveUnits($theFullWaveName, 3 ) + ";"
		
		// add wave note	
		theWaveNote +=  ";"  + note($(inDataFolder + "'" + theWaveName + "'"))
					
		Variable theKeyIndex
	
		// match key loop
		for (theKeyIndex = 0; theKeyIndex <  theNumberofMatchKeys; theKeyIndex += 1)

			String theItem = StringFromList(theKeyIndex, inMatchKeyList)

			String theItemList = ListMatch(theWaveNote, theItem)
			
			// check match
			if (strlen(theItemList) == 0)
				
				// set index
				theKeyindex = -1
				
				// no match, exit loop
				break
			endif
		endfor
		
		// all keys matched?
		if (theKeyIndex != -1)
		
			// add wave name
			theWaveList += inDataFolder + "'" +theWaveName + "'" + ";"
		endif
	endfor
		
	Variable theNumberOfObjects = CountObjects(inDataFolder, 4)
		
	for (i = 0; i < theNumberOfObjects ; i+=1 )
		
		// get folder
		String theFolderName = GetIndexedObjName(inDataFolder, 4, i)
		
		// get sub folder name
		String theSubFolderName = inDataFolder + "'" + theFolderName + "'"+ ":"

		theWaveList += DoRecursiveWaveSearch(theSubFolderName, inMatchKeyList) 

	endfor
	
	// return list
	return theWaveList
End

Function/S GetProjectWaveList(inProjectName, inMatchKeyList)

	String inProjectName, inMatchKeyList
	
	String theProjectDF = GetStandardProjectData(inProjectName, "DataFolder")
	
	String theWaveList = ""
	
	if (strlen(theProjectDF) > 0)
	
		theWaveList = DoRecursiveWaveSearch(theProjectDF + ":" ,inMatchKeyList)
	endif

	return theWaveList
End

Function ModityTacesInGraphs(inModKey, inValueString, inGraphList)

	String inModKey, inValueString, inGraphList

	String theSelection = GetSelectedWaves()
	
	Variable theNumberOfWaves = ItemsInList(theSelection)
	Variable i
	String theDisplayedWaves = ""
		
	for (i = 0; i < theNumberOfWaves; i+= 1)
		
		String theWave = StringFromList(i, theSelection)
		
		CheckDisplayed /A $theWave
		 
		if (V_Flag == 1)
			theDisplayedWaves += GetWavesDataFolder($theWave, 2 ) + ";"
		endif
	endfor

	String theGraphList = inGraphList

	Variable theNumberOfGraphWindows = ItemsInList(theGraphList)
	
	Variable theGraphIndex
	
	for (theGraphIndex = 0; theGraphIndex < theNumberOfGraphWindows; theGraphIndex += 1)

		String theGraphName = StringFromList(theGraphIndex, theGraphList)
		
		Variable theTraceIndex
		
		String theTraces = TraceNameList(theGraphName, ";", 1)

		Variable theNumberOfTraces = ItemsInList(theTraces)

		for (theTraceIndex = 0; theTraceIndex < theNumberOfTraces; theTraceIndex += 1)

			String theTraceName = StringFromList(theTraceIndex, theTraces)
			
			Wave theWaveRef = TraceNameToWaveRef(theGraphName, theTraceName)
			
			String theWaveName = GetWavesDataFolder(theWaveRef, 2 )
			
			if (WhichListItem(theWaveName, theDisplayedWaves) > -0.5)
			
				// hide trace
				Execute /Q" ModifyGraph /W=" + theGraphName + " " + inModKey + "("+theTraceName+")=" +inValueString
			endif
		endfor
	endfor
End

Function OneGraphExists()

	// get top graph name
	String theTopGraph = WinName(0, 1)

	if (strlen(theTopGraph) > 0)
		return 1
	else
		return 0
	endif
End

Function isProjectWave(inWave)

	String inWave
	
	String theNote = Note($inWave)
	
	String theKey = StringByKey("isProjectWave",theNote, "=", ";")
	
	if (stringmatch(theKey,"True"))
		return 1
	else
		return 0
	endif
End

Function ExportGraphs(inExportGraphList)

	String inExportGraphList

	Variable theNumberOfGraphs = ItemsInList(inExportGraphList)
	
	Variable theGraphIndex
	
	String theExportPathName = "ExportProjectGraphPath"

	if (WhichListItem(theExportPathName, PathList("*", ";", "")) > -1)
	
		// cleanup
		KillPath $theExportPathName
	endif
		
	// graph loop
	for (theGraphIndex = 0; theGraphIndex < theNumberOfGraphs; theGraphIndex += 1)
	
		// extract item
		String theGraphName = StringFromList(theGraphIndex, inExportGraphList)
		
		String theFileName = CleanupName(GetNamedWindowTitle(theGraphName), 0) + "_"+ theGraphName + ".emf"
		
		if (strlen(theFileName) == 0)
			theFileName = theGraphName
		endif 
			
		// get path list
		String thePathList = PathList("*", ";", "")

		if (WhichListItem(theExportPathName, thePathList) == -1)
			
			// set export path
			NewPath/Q /O $theExportPathName
		endif	
			
		// save graph
		SavePICT /E=-2 /P=$theExportPathName /N=$theGraphName /O /Q=1.0 as theFileName
	endfor
	
	if (WhichListItem(theExportPathName, PathList("*", ";", "")) > -1)
		
		// cleanup
		KillPath $theExportPathName
	endif	
End

Function/S DoProjectReport(inProjectName, inReportKeyList, inGraphPropertyList, inGraphSeperatorString )

	String inProjectName, inReportKeyList, inGraphPropertyList, inGraphSeperatorString 

	// new graph windows
	String theGraphList = ""
	
	Variable theIndex
	
	// match key loop
	for (theIndex = 0; theIndex < ItemsInList(inReportKeyList, inGraphSeperatorString); theIndex += 1)
	
		// extract match key
		String theMatchKeyList = StringfromList(theIndex,inReportKeyList, inGraphSeperatorString)
	
		// get wave list
		String theWaveList = DoRecursiveWaveSearch("root:", theMatchKeyList)
		
		// build graph key list
		String theGraphKeys = inGraphPropertyList + "isPartOfSeries=True;GraphNumber=" + num2str(theIndex) + ";"
		
		// open new simdrive graph
		String theGraphName = NewProjectWindow(inProjectName , theGraphKeys,1) 
		
		// display list
		DisplayProjectWaves(theGraphName, theWaveList)
		
		// do style
		String theGraphStyleFunctionName = GetStandardProjectData(inProjectName, "GraphStyleFunction")
		
		// execute graph style function
		Execute theGraphStyleFunctionName + "(\"" + theGraphName+ "\")"
				
		// kill graph without traces
		if (ItemsInList(TraceNameList(theGraphName, ";" , 1)) == 0)
			
			// kill !
			DoWindow /K $theGraphName
		else
		
			// add name to graph list
			theGraphList += theGraphName + ";" 
		endif
	endfor
	
	// return created graphs
	return theGraphList
End

Function compareDataFolders(inFolder1, inFolder2)

	String inFolder1, inFolder2
	
	Variable theNumberOfItens = max(ItemsInList(inFolder1, ":"), ItemsInList(inFolder2, ":"))
	
	Variable theLevel
	
	for (theLevel = theNumberOfItens; theLevel > 0; theLevel -= 1)
	
		String theDataFolder1 = StringFromList(theLevel, inFolder1, ":")

		String theDataFolder2 = StringFromList(theLevel, inFolder2, ":")

		if (stringmatch(theDataFolder1[0], "'") == 1)
			theDataFolder1 = StringFromList(1, theDataFolder1, "'") 
		endif

		if (stringmatch(theDataFolder2[0], "'") == 1)
			theDataFolder2 = StringFromList(1, theDataFolder2, "'") 
		endif
	
		if (stringmatch(theDataFolder1, theDataFolder2) == 0)
			return 0
		endif
	endfor

	return 1
End

Function ProjectGraphStyle(inGraphName)

	String inGraphName
	
	String theTraceNameList = TraceNameList(inGraphName, ";" , 1)
	
	String theLegendString = ""
	
	// are there traces?
	if (ItemsInList(theTraceNameList) > 0)
	
		Variable theMaximumNumberOfMarkers = 10
	
		// trace loop
		Variable theTraceIndex
		for  (theTraceIndex = 0; theTraceIndex < ItemsInList(theTraceNameList); theTraceIndex += 1)

			// get trace name
			String theTraceName = StringFromList(theTraceIndex, theTraceNameList)
		
			Wave theWaveRef = TraceNameToWaveRef(inGraphName, theTraceName)
		
			// get note
			String theNote = note(theWaveRef)
		
			Variable theMSkipValue = ceil(numpnts(theWaveRef)/theMaximumNumberOfMarkers)
			
			// get marker
			String theMarker = StringByKey("Marker", theNote, "=", ";")
			
			if (strlen(theMarker) > 0)
			
				// set maximum number of markers
				ModifyGraph /W = $inGraphName mskip($theTraceName)=theMSkipValue
			endif
			
			// build legend string
			theLegendString += "\\s("+ theTraceName + ") " + StringByKey("Legend", theNote , "=" , ";") +  "\r"
		endfor
		
		Legend /K /N=ProjectLegend /W=$inGraphName 
		
		// set legend
		Legend /N=ProjectLegend /A=RC /E=1 /W=$inGraphName theLegendString[0,strlen(theLegendString)-2]

		// execute command
		ModifyGraph /W=$inGraphName minor(left)=1
		ModifyGraph /W=$inGraphName minor=1
		ModifyGraph /W=$inGraphName grid(left)=2
		ModifyGraph /W=$inGraphName grid(bottom)=2
				
		String theAxisList = AxisList(inGraphName)
		
		Variable theAxisIndex
		
		// axis loop
		for (theAxisIndex = 0; theAxisIndex < ItemsInList(theAxisList); theAxisIndex += 1)
		
			// get axis
			String theAxis = StringFromList(theAxisIndex, theAxisList)
		
			// set axis
			SetAxis /W=$inGraphName /A/N=1 $theAxis
		endfor
		
		// set legend (take legend from first trace)
		// TODO: check legend match of all traces!
		String theYLegend = StringByKey("Legend" , theNote, "=", ";")
				
		// x label
		String theXLabel = StringByKey("XAxisLabel", theNote, "=", ";")
		Label bottom theXLabel
		
		// y label
		String theYLabel = StringByKey("YAxisLabel", theNote, "=", ";")	
		Label left theYLabel

		// show info
		ShowInfo /W=$inGraphName
		
		// TODO: check units!
	endif
End

Function SetCurrentProject(inProjectName)

	String inProjectName
	
	String theDataFolder = GetStandardProjectData(inProjectName, "DataFolder")
	
	if (DataFolderExists(theDataFolder) == 1)
	
		SetDataFolder $theDataFolder
	endif

	return 0
End

Function/S GetStandardObjectColor(inObject, inObjectList, inColorTable ) 

	String inObject, inObjectList, inColorTable

	// get color table data
	ColorTab2Wave $inColorTable
	Wave theColorWave = M_Colors
		
	// sort list
	inObjectList = SortList(inObjectList)
	
	// count items
	Variable theNumberOfObjects = ItemsInList(inObjectList,";")
				
	// find current wave in list
	Variable theObjectIndex = WhichListItem(inObject,inObjectList) 

	// calc color index
	Variable theIndex = theObjectIndex/(theNumberOfObjects-1) * (DimSize(theColorWave,0)-1)
						
	// return r,g,b
	String theColor = num2str(theColorWave[theIndex][0]) + "," + num2str(theColorWave[theIndex][1]) + "," + num2str( theColorWave[theindex][2])		

	// cleanup
	KillWaves /Z theColorWave

	return theColor
End

Function/S ListWaveSearch(inWaveList, inMatchKeys)

	String inWaveList, inMatchKeys

	String OutWave =""
	
	Variable i , j

	for (i = 0 ; i < ItemsInList(inWaveList) ; i+=1)

		String theItem = StringFromList(i,inWaveList,";")
		String theNote =note($theItem)
		
		for(j = 0 ; j < ItemsInList(inMatchKeys) ; j+=1)
		
			String theMatch = StringFromList(j,inMatchKeys,";")
			
			String theMatchKey = StringFromList(0,theMatch,"=")
			String theMatchValue = StringFromList(1,theMatch,"=")
			
			String theValue = StringByKey(theMatchKey , theNote , "=" , ";")
			
			if( !StringMatch(theValue,theMatchValue))
				break
			endif
		endfor
		
		if(j == ItemsInList(inMatchKeys))
			OutWave += theItem+";"
		endif
	endfor
	
	return OutWave
End

Function/S ListWaveValueList(inWaveList, inKey)

	String inWaveList, inKey

	String OutList =""
	
	Variable i , j

	for (i = 0 ; i < ItemsInList(inWaveList) ; i+=1)

		String theItem = StringFromList(i,inWaveList,";")

		String theValue = GetWaveKey(theItem, inKey)
		
		if (strlen(theValue) > 0 && WhichListItem(theValue, OutList) == -1)

			// add value
			OutList += theValue+";"
		endif
	endfor
	
	return OutList
End

Function/S ReadTextFile(inTextFile)

	String inTextFile
	
	// open notebook
	OpenNotebook /K=1 /N=theFile /V=0 inTextFile
	
	// select data
	Notebook theFile selection={startOfFile, endOfFile }
	
	GetSelection notebook, theFile , 2
	
	// copy host names
	String theData = S_Selection
	
	// close host file
	DoWindow /K  theFile
	
	// return
	return theData
End

Function ProjectGraphHook(H_Struct)
	STRUCT WMWinHookStruct &H_Struct

	if (H_Struct.eventCode == 8)
	
		String theWindowName = H_Struct.winName
	
		String theWinRecreation = WinRecreation(theWindowName, 0 )
		
		String theLeftLabelList = ListMatch(theWinRecreation, "	Label left*", "\r")		
				
		String theFirstLeftLabel = StringFromList(0, theLeftLabelList)
		
		String theLeftLabel = StringFromList(1, theFirstLeftLabel, "\"")		
		theLeftLabel = StringFromList(0, theLeftLabel, "\"")		
				
		String theBottomLabelList = ListMatch(theWinRecreation, "	Label bottom*", "\r")		
		
		String theFirstBottomLabel = StringFromList(0, theBottomLabelList)
		
		String theBottomLabel = StringFromList(1, theFirstBottomLabel, "\"")		
		theBottomLabel = StringFromList(0, theBottomLabel, "\"")		

		DoWindow/T $theWindowName,  theLeftLabel + " vs " + theBottomLabel 
	
		return 1
	endif

	return 0
End

Function/S GetNamedWindowTitle(windowNameStr)	
	
	String windowNameStr	
	String returnStr="",recreationStr=WinRecreation(windowNameStr,0)
	Variable startPos,stopPos	
	if (!((strlen(windowNameStr)>0) && (cmpstr(WinList(windowNameStr,"",""),"")==0)))		
	
	startPos=strsearch(recreationStr," as \"",0) // search for 'Display /W(...) as "'		

	if (startPos>0)			
	
		stopPos=strsearch(recreationStr,"\"",startPos+5)			
	
		if (stopPos>0)				
			returnStr=recreationStr[startPos+5,stopPos-1]			
		endif	
	else			
			returnStr=WinName(0,1+2+4+16+64)		
		endif	
	endif	
	
	return returnStr
End